<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
require_once(APPPATH.'classes/ci_extend_libraries/ControllerBase.abstract.php');	

/**
 * 
 */
class PayProcessException extends Exception
{
	const UNKNOWN				 		= 0;
	const FAIL_WRITE_REQUEST_LOG 		= 101;
	const FAIL_WRITE_SUBMIT_LOG 		= 102;
	const FAIL_WRITE_RESULT_LOG 		= 103;
	const FAIL_WRITE_RESPONSE_LOG 		= 104;
	const FAIL_WRITE_TRANSACTION_LOG 	= 105;
	const FAIL_WRITE_TRANSACTION_TRN 	= 106;

	/**
	 * 
	 */
	public function __construct($code = self::UNKNOWN)
	{
		parent::__construct(null, $code);	
	}
}

/**
 * 
 */
abstract class PaymentControllerBase extends ControllerBase 
{
	/**
	 * コンストラクタ
	 */
	public function __construct()
	{
		parent::__construct();
		// モデル先読み（微妙 orz）
		self::_getTableModel('TransactionLogModel');
		self::_getTableModel('TransactionTrnModel');
	}
	/**
	 * TODO2: 適切な実装にしないと "transaction_trn" がエラー吐く
	 * @return bool 成功時は true
	 */
	abstract protected function _onSubmit(&$paydrv, &$obj_req);
	/**
	 * @return string 'models/transaction/' 配下のモデル名
	 */
	abstract protected function _getHttpRequestParserModelName();

	/**
	 * 
	 */
	protected function _execTransaction()
	{
		try
		{
			//
			$res = null;
			// Request パース
			$HTTPREQ =& $this->_onCreateHttpRequestModel();
			$HTTPREQ->parseRequest();
			$obj_req = $HTTPREQ->getRequestObject();
			// リザルトデータ格納オブジェクト
			$obj_res = new TransactionResultData();

			do
			{
				// Request ログ記録
				$body = $this->input->server('REQUEST_URI', true) . '?' . $obj_req->toNamevalueString();
				$trnid = $this->_writeRequestLog($body, $this->input->ip_address());
				// Validate
				if(!$HTTPREQ->isValidate())
				{
					$obj_res->RESULT = current($HTTPREQ->getErrors());
					break;
				}
				
				// Payment モデル初期化 ＆Payment Driver 呼び出し
				$PAYMENT =& $this->_getPaymentModel();
				$res = $PAYMENT->loginMerchant($obj_req->MERCHANTID, $obj_req->MERCHANTPW);
				if(!$res)
				{
					$obj_res->RESULT = TransactionResultData::USER_AUTHENTICATION_FAILED;
					break;
				}
				$PAYDRV =& $PAYMENT->getPaymentDriver();

				// ORIGID を場合は検索
				$res = $this->_margeTransactionTrnValue($obj_req);
				if(!$res)
				{
					$obj_res->RESULT = TransactionResultData::TRANSACTIONID_NOTFOUND;
					break;
				}

				// Submit
				$trans_proc_id = $this->_onSubmit($PAYDRV, $obj_req);
				if(!$trans_proc_id)
				{
					$obj_res->RESULT = TransactionResultData::SUBMIT_EXECUTE_FAILED;
					break;
				}

				// Submit ログ記録
				$body = $PAYDRV->getSubmitValue();
				$this->_writeSubmitLog($trnid, $body);
				// Response ログ記録
				$body = $PAYDRV->getResponseValue();
				$this->_writeResponseLog($trnid, $body);

				// Response → Result 変換
				$obj_rps = $PAYDRV->getResponseObject();
				$obj_res->RESULT	= $obj_rps->getPublicResultCode();
				$obj_res->RESPMSG	= $obj_rps->getPublicResultMessage();
				$obj_res->PNREF		= $PAYDRV->getTransactionId();

				// トランザクションログ記録
				$this->_writeTransactionTrn($PAYDRV->getTransactionId(), $obj_rps->getOriginalTransactionCode(), $trans_proc_id, $PAYMENT->getMerchantInfo('ID'));
			}
			while(false);

			// Result ログ記録
			$result = $obj_res->toNamevalueString();
			$this->_writeResultLog($trnid, $result);

			// Result
			echo $result;
		}
		catch(PayProcessException $e)
		{
			// TODO2: 死すログ
			echo $e;
		}
		catch(Exception $e)
		{
			// TODO2: 死すログ
			echo $e;
		}
	}
	/**
	 * 
	 */
	protected function& _onCreateHttpRequestModel()
	{
		$name = $this->_getHttpRequestParserModelName();
		$this->load->model('transaction/' . $name);
		$model =& $this->$name;
		return $model;
	}
	/**
	 * 
	 */
	protected function& _getPaymentModel()
	{
		// Payment モデル呼び出し
		$this->load->model('payment/payment');
		return $this->payment;
	}
	/**
	 * 
	 */
	protected function _onGetParseHttpRequest(&$model_httpreq)
	{
		// パース＆値検証
		$model_httpreq->parseRequest();
		// 値オブジェクトを取得
		return $model_httpreq->getRequestObject();
	}
	/**
	 * 
	 */
	protected function _margeTransactionTrnValue($obj_req)
	{
		// 含まれていない場合は復帰
		if(!isset($obj_req->ORIGID))
			return $obj_req;
		// 検索
		$row_trans = $this->_getTransactionTrn($obj_req->ORIGID);
		if(!$row_trans)
			return null;
		// リクエストオブジェクトにマージ
		$obj_req->parseArray($row_trans);
		$obj_req->ORIGID = $obj_req->ORIG_TRN_ID;	// payment のIDで上書き
		//
		return $obj_req;
	}

	/**
	 * 
	 */
    protected function _writeTransactionLog($logenum, $logid, $trnid)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionLogModel');
		$res = $MODEL->writeLogId($logenum, $logid, $trnid);
		if(empty($res))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_TRANSACTION_LOG);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _writeRequestLog($body, $address)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('RequestLogModel');
		$logid = $MODEL->writeLog($body, $address);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_REQUEST_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_REQUEST, $logid, null);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _writeSubmitLog($trnid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('SubmitLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_SUBMIT_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_SUBMIT, $logid, $trnid);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _writeResponseLog($trnid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('ResponseLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESPONSE_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_RESPONSE, $logid, $trnid);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _writeResultLog($trnid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('ResultLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESULT_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_RESULT, $logid, $trnid);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _writeTransactionTrn($trnid, $orgtrnid, $processed, $merchantid)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionTrnModel');
		$res = $MODEL->writeValue($trnid, $orgtrnid, $processed, $merchantid);
		if(empty($res))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESULT_LOG);
		// 復帰
		return $res;
    }
	/**
	 * 
	 */
    protected function _getTransactionTrn($trnid)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionTrnModel');
		$res = $MODEL->getValue($trnid);
		// 復帰
		return $res;
    }
    /**
     * 
     */
    protected static function& _getTableModel($name)
    {
		$CI =& get_instance();
		$CI->load->model('database/tables/' . $name, '', true);
		return $CI->$name;
    }
	
}

/* End of file */